@@ -1,39 +1,39 @@
-from typing import List, Optional, Callable, Union
+from typing import List, Union, Dict
 
 import numpy as np
 import pandas as pd
 from sklearn.base import BaseEstimator
 
-from deduplipy.blocking.blocking_rules import *
 from deduplipy.blocking.set_cover import greedy_set_cover
 from deduplipy.config import ROW_ID
 
 
 class Blocking(BaseEstimator):
-    def __init__(self, col_names: Union[List[str], str], rules: Optional[List[Callable]] = None, recall: float = 1.0,
+    def __init__(self, col_names: List[str], rules_info: Dict, recall: float = 1.0,
                  save_intermediate_steps: bool = False) -> 'Blocking':
         """
         Class for fitting blocking rules and applying them on new pairs
 
         Args:
-            col_names: column names on which blocking rules should be applied
-            rules: list of rules to test for blocking, if None, all available blocking rules are used
+            col_names: list of column names, also the ones not included in blocking
+            rules_info: dict with column names as keys and a list of blocking functions as values
             recall: minimum recall required
             save_intermediate_steps: whether or not to save intermediate results
 
         """
-        if isinstance(col_names, list):
-            self.col_names = col_names
-        elif isinstance(col_names, str):
-            self.col_names = [col_names]
-        else:
-            raise Exception('col_name should be list or string')
-        self.rules = rules
-        if not self.rules:
-            self.rules = all_rules
+        self.rules_info = rules_info
+        self.col_names = col_names
+        self._get_rules_specs()
         self.recall = recall
         self.save_intermediate_steps = save_intermediate_steps
 
+    def _get_rules_specs(self):
+        self._rules_specs = []
+        for col_name, functions in self.rules_info.items():
+            for function in functions:
+                self._rules_specs.append(
+                    {'col_name': col_name, 'function': function, 'function_name': function.__name__})
+
     def fit(self, X: Union[pd.DataFrame, np.ndarray], y: Union[pd.DataFrame, np.ndarray]) -> 'Blocking':
         """
         Fit Blocking instance on data
@@ -56,26 +56,30 @@ class Blocking(BaseEstimator):
             else:
                 return result + f":{j}"
 
-        for j, rule in enumerate(self.rules):
-            for col_name in self.col_names:
-                df_training[f'{col_name}_1_{rule.__name__}'] = df_training.apply(
-                    lambda row: apply_rule(rule, row[f'{col_name}_1'], j), axis=1)
-                df_training[f'{col_name}_2_{rule.__name__}'] = df_training.apply(
-                    lambda row: apply_rule(rule, row[f'{col_name}_2'], j), axis=1)
-                df_training[f'{col_name}_{rule.__name__}'] = df_training.apply(
-                    lambda row: int(((row[f'{col_name}_1_{rule.__name__}'] != None) &
-                                     (row[f'{col_name}_2_{rule.__name__}'] != None)) &
-                                    (row[f'{col_name}_1_{rule.__name__}'] == row[f'{col_name}_2_{rule.__name__}'])),
-                    axis=1)
+        for j, rule_spec in enumerate(self._rules_specs):
+            col_name = rule_spec['col_name']
+            func_name = rule_spec['function_name']
+            function = rule_spec['function']
+            df_training[f'{col_name}_1_{func_name}'] = df_training.apply(
+                lambda row: apply_rule(function, row[f'{col_name}_1'], j), axis=1)
+            df_training[f'{col_name}_2_{func_name}'] = df_training.apply(
+                lambda row: apply_rule(function, row[f'{col_name}_2'], j), axis=1)
+            df_training[f'{col_name}_{func_name}'] = df_training.apply(
+                lambda row: int(((row[f'{col_name}_1_{func_name}'] != None) &
+                                 (row[f'{col_name}_2_{func_name}'] != None)) &
+                                (row[f'{col_name}_1_{func_name}'] == row[f'{col_name}_2_{func_name}'])),
+                axis=1)
+            self._rules_specs[j].update({'rule_set': set(
+                df_training[df_training[f'{col_name}_{func_name}'] == 1][f'{col_name}_{func_name}'].index.tolist())})
 
-        rule_sets = dict()
-        for rule in self.rules:
-            for col_name in self.col_names:
-                rule_sets.update(
-                    {f'{col_name}_{rule.__name__}': [rule, rule.__name__, col_name, set(
-                        df_training[df_training[f'{col_name}_{rule.__name__}'] == 1][
-                            f'{col_name}_{rule.__name__}'].index.tolist())]})
-        self.subsets = [x[3] for x in rule_sets.values()]
+        # rule_sets = dict
+        # for rule in self.rules:
+        #     for col_name in self.col_names:
+        #         rule_sets.update(
+        #             {f'{col_name}_{rule.__name__}': [rule, rule.__name__, col_name, set(
+        #                 df_training[df_training[f'{col_name}_{rule.__name__}'] == 1][
+        #                     f'{col_name}_{rule.__name__}'].index.tolist())]})
+        self.subsets = [x['rule_set'] for x in self._rules_specs]
 
         self.matches = df_training[df_training.match == 1].index.tolist()
         self.universe = set(self.matches)
@@ -83,10 +87,9 @@ class Blocking(BaseEstimator):
         self.cover = greedy_set_cover(self.subsets, self.universe, self.recall)
 
         self.rules_selected = []
-        for rule_name, rule_specs in rule_sets.items():
-            rule, rule_name, col_name, rule_set = rule_specs
-            if rule_set in self.cover:
-                self.rules_selected.append([rule, rule_name, col_name])
+        for rule_spec in self._rules_specs:
+            if rule_spec['rule_set'] in self.cover:
+                self.rules_selected.append(rule_spec)
         return self
 
     def _fingerprint(self, X: pd.DataFrame) -> pd.DataFrame:
@@ -102,10 +105,12 @@ class Blocking(BaseEstimator):
         """
         df = X.copy()
         for j, rule_selected in enumerate(self.rules_selected):
-            rule, rule_name, col_name = rule_selected
-            df[f'{col_name}_{rule_name}'] = df[col_name].apply(lambda x: rule(x))
-            df.loc[df[f'{col_name}_{rule_name}'].notnull(), f'{col_name}_{rule_name}'] = \
-                df[df[f'{col_name}_{rule_name}'].notnull()][f'{col_name}_{rule_name}'] + f":{j}"
+            col_name = rule_selected['col_name']
+            func_name = rule_selected['function_name']
+            function = rule_selected['function']
+            df[f'{col_name}_{func_name}'] = df[col_name].apply(lambda x: function(x))
+            df.loc[df[f'{col_name}_{func_name}'].notnull(), f'{col_name}_{func_name}'] = \
+                df[df[f'{col_name}_{func_name}'].notnull()][f'{col_name}_{func_name}'] + f":{j}"
         df_melted = df.melt(id_vars=self.col_names + [ROW_ID], value_name='fingerprint').drop(columns=['variable'])
         df_melted.dropna(inplace=True)
         return df_melted
